Attribute VB_Name = "Primitives"
''''''''''''''''''
''' Primitives '''
''''''''''''''''''

'' These functions write out the lowest-level entities understood
'' by BZFlag.

''' ORIGINALLY, these wrote directly to file.  Now they're writing
''' to a queue, which will be processed separately.  The main motivation
''' for this is to allow complicated compound objects to be defined and
''' manipulated as collections of primitives

Sub world(size)
    If Not V2 Then Exit Sub
    raw_emit "world"
    raw_emit "  size " & CStr(size)
    raw_emit "end"
End Sub

Sub install_options()
    Dim i As Integer
    If Form1.BZVersion = "BZFlag1.10.8" Then Exit Sub
    If NumWorldOptions = 0 Then Exit Sub
    raw_emit "options"
    For i = 0 To NumWorldOptions
        raw_emit "  -set " & WorldOptions(i) & " " & WorldValues(i)
    Next i
    raw_emit "end"
End Sub

Sub Base(x, y, z, sizeX, sizeY, sizeZ, theta, color)
    '' COLOR: 1=Red, 2=Blue, 3=Green, 4=Purple
    '' -mp flag on bzfs must have matching counts, else server error.
    If CurrentQueue = 0 Then numbases = numbases + 1
    QueueBZObject "base", x, y, z, theta, sizeX, sizeY, sizeZ, color, "BASE"
End Sub


Sub Box(x, y, z, rotation, sizeX, sizeY, sizeZ, Optional name As String)
    If CurrentQueue = 0 Then NumBoxes = NumBoxes + 1
    If sizeX = 0 Or sizeY = 0 Then
        MsgBox "Error - size is zero!"
        Error 1
    End If
    QueueBZObject "box", x, y, z, rotation, sizeX, sizeY, sizeZ, 0, name, False, False, False
End Sub

Sub Pyramid(x, y, z, rotation, sizeX, sizeY, sizeZ, _
       Optional ShootThrough As Boolean = False, _
       Optional DriveThrough As Boolean = False, _
       Optional Flipz As Boolean = False, _
       Optional name As String)
    If CurrentQueue = 0 Then NumPyramids = NumPyramids + 1
    QueueBZObject "pyramid", x, y, z, rotation, sizeX, sizeY, sizeZ, 0, name, ShootThrough, DriveThrough, Flipz
End Sub

Sub Teleporter(x, y, z, rotation, sizeX, sizeY, sizeZ, sizeB, Optional tName = "Noname")
    Dim istart As Integer
    Dim iend As Integer
    NumTeleporters = NumTeleporters + 1
    istart = (NumTeleporters - 1) * 2
    iend = istart + 1
    TeleporterName(NumTeleporters) = tName
    TeleporterFront(NumTeleporters) = istart
    TeleporterBack(NumTeleporters) = iend
    QueueBZObject "teleporter", x, y, z, rotation, sizeX, sizeY, sizeZ, sizeB, tName
    '(Teleporter counting is done at a higher level, as part of managing links.)
End Sub

Sub QueueBZObject(OType As String, x, y, z, rotation, _
        sizeX, sizeY, sizeZ, sizeB, name, _
        Optional sTh As Boolean = False, _
        Optional dTh As Boolean = False, _
        Optional Flipz As Boolean = False)
    Dim NameArg As String
    If Not IsMissing(name) Then
        NameArg = name
    Else
        NameArg = ""
    End If
    
    If sizeX = 0 Or sizeY = 0 Then
        MsgBox "Queuing object with zero size."
        Error 1
    End If
    
    'Write to queue(currentqueue)
    Dim gobj As New BZFlagObject
    gobj.ObjectType = OType
    gobj.x = x
    gobj.y = y
    gobj.z = z
    gobj.theta = rotation
    gobj.xSize = sizeX
    gobj.ySize = sizeY
    gobj.zSize = sizeZ
    gobj.bsize = sizeB
    gobj.name = NameArg
    gobj.ShootThrough = sTh
    gobj.DriveThrough = dTh
    gobj.Flipz = Flipz
    
    GQueue(CurrentQueue).AddEntry gobj
End Sub

'' Must Queue up the raw writes too.
Sub emit(inarg As String)
    'Write to queue(currentqueue)
    Dim gobj As New BZFlagObject
    gobj.ObjectType = "String"
    gobj.x = 0
    gobj.y = 0
    gobj.z = 0
    gobj.theta = 0
    gobj.xSize = 0
    gobj.ySize = 0
    gobj.zSize = 0
    gobj.name = inarg
    GQueue(CurrentQueue).AddEntry gobj
End Sub


Sub comment(inarg As String)
    '' Format with the comment prefix; also useful in making
    '' the build script clearer.
    NumComments = NumComments + 1
    emit "# " & inarg
End Sub

'''''''''''''''''''''''''''''''''''''
''' Handling (Processing) A Queue '''
'''''''''''''''''''''''''''''''''''''

Sub analyze_q(inq As Queue)
    Dim i As Long
    Dim BZtype As String
    Dim BZobj As BZFlagObject
    Dim nstrings, nBoxs, npyramids, nteleporters, nelse, nbases
    Dim tstr As String
    Form1.Qsize = inq.NumEntries
    'Open "str.txt" For Output As #3
    For i = inq.FirstEntry To inq.NumEntries
        On Error GoTo trynext
        Set BZobj = inq.GetEntry(i)
        BZtype = BZobj.ObjectType
        
        Select Case LCase(BZtype)
            Case "string"
                'Print #3, BZobj.Name
                nstrings = nstrings + 1
            Case "box"
                nBoxs = nBoxs + 1
            Case "pyramid"
                npyramids = npyramids + 1
            Case "teleporter"
                nteleporters = nteleporters + 1
            Case "base"
                nbases = nbases + 1
            Case Else
                nelse = nelse + 1
        End Select
trynext:
    Next i
    'Close #3
'    tstr = "Queue Analysis (" & CStr(inq.NumEntries) & " items): " & vbCr
'    tstr = tstr & "Strings: " & CStr(nStrings) & vbCr
'    tstr = tstr & "Boxes: " & CStr(nBoxs) & vbCr
'    tstr = tstr & "Pyramids: " & CStr(nPyramids) & vbCr
'    tstr = tstr & "Teleporters: " & CStr(nTeleporters) & vbCr
'    tstr = tstr & "Other (unknown): " & CStr(nElse) & vbCr
'    tstr = tstr & " Total: " & CStr(nStrings + nBoxs + nPyramids + nTeleporters + nElse)
'    raw_emit tstr
    Form1.nstrings = nstrings
    Form1.nboxes = nBoxs
'    NumBoxes = nBoxs
    
    Form1.npyramids = npyramids
'    NumPyramids = npyramids
    
    Form1.nteleporters = nteleporters
    Form1.TotalRendered = nBoxs + npyramids + nteleporters
    Form1.ntotal = nstrings + nBoxs + npyramids + nteleporters + nelse
    Form1.nUnknown = nelse
    Form1.nDirect = NumDirect
    Form1.nlinks = NumLinks
End Sub

Sub ProcessQueue(inq As Queue)
    Dim i As Long
    Dim BZobj As BZFlagObject
    Dim BZtype As String
    Dim x, y, z, theta, xSize, ySize, zSize
    Dim name As String
    Dim sTh As Boolean
    Dim dTh As Boolean
    Dim Flipz As Boolean
    
    'First, analyze Q for debugging
    analyze_q inq
    emit_File_Info
    
    install_options
    
    'Reset direct count (we don't care when we're here)
    NumDirect = 0
    
    For i = inq.FirstEntry To inq.NumEntries
        
        'On Error GoTo trynext
            Set BZobj = inq.GetEntry(i)
            
            'Destructure it.
            BZtype = BZobj.ObjectType
            name = BZobj.name
            x = BZobj.x
            If ReflectX Then
                x = -x
            End If
            
            y = BZobj.y
            If ReflectY Then
                y = -y
            End If
            
            z = BZobj.z
            theta = BZobj.theta
            If ReflectX Then
                theta = 180 - theta
            End If
            If ReflectY Then
                theta = -theta
            End If
            
            xSize = BZobj.xSize
            ySize = BZobj.ySize
            zSize = BZobj.zSize
'            If ReflectX Then
'                Xsize = -Xsize
'            End If
'            If ReflectY Then
'                Ysize = -Ysize
'            End If
            
            bsize = BZobj.bsize
            sTh = BZobj.ShootThrough
            dTh = BZobj.DriveThrough
            Flipz = BZobj.Flipz
            
            ' Dispatch and draw it.
            Select Case LCase(BZtype)
                Case "string"
                    raw_emit name
                Case "base"
                    pBase x, y, z, theta, xSize, ySize, zSize, bsize
                Case "box"
                    pBox x, y, z, theta, xSize, ySize, zSize
                Case "pyramid"
                    pPyramid x, y, z, theta, xSize, ySize, zSize, sTh, dTh, Flipz
                Case "teleporter"
                    pTeleporter x, y, z, theta, xSize, ySize, zSize, bsize, name
            End Select
trynext:
    Next i
End Sub


''''''''''''''''''''''''''
''' Drawing Primitives '''
''''''''''''''''''''''''''

'' First, a formatting utility: restrict precision to 2 decimal places.
Function nArg(inNumber) As String
    nArg = Format(inNumber, "#0.0#")
    If nArg = CStr(inNumber) Then
        nArg = CStr(inNumber)
    End If
End Function

'For integeres
Function vArg(inNumber) As String
    vArg = Format(inNumber, "###0")
    If vArg = CStr(inNumber) Then
        vArg = CStr(inNumber)
    End If
End Function
'' Bases
Sub pBase(x, y, z, theta, sizeX, sizeY, sizeZ, color)
    raw_emit "base"
    raw_emit "  position " & nArg(x) & " " & nArg(y) & " " & nArg(z)
    raw_emit "  rotation " & nArg(theta)
    raw_emit "  size     " & nArg(sizeX) & " " & nArg(sizeY) & " " & nArg(sizeZ)
    raw_emit "  color    " & CInt(color)
    raw_emit "end"
End Sub

'' The basic box.
Sub pBox(x, y, z, rotation, sizeX, sizeY, sizeZ)
    raw_emit "box"
    raw_emit "  position " & nArg(x) & " " & nArg(y) & " " & nArg(z)
    raw_emit "  rotation " & nArg(rotation)
    raw_emit "  size     " & nArg(sizeX) & " " & nArg(sizeY) & " " & nArg(sizeZ)
    raw_emit "end"
End Sub

'' The basic pyramid.
Sub pPyramid(x, y, z, rotation, sizeX, sizeY, sizeZ, sTh, dTh, Flipz)
    raw_emit "pyramid"
    raw_emit "  position " & nArg(x) & " " & nArg(y) & " " & nArg(z)
    raw_emit "  rotation " & nArg(rotation)
    raw_emit "  size     " & nArg(sizeX) & " " & nArg(sizeY) & " " & nArg(sizeZ)
    If sTh Then
        raw_emit "  ShootThrough"
    End If
    If dTh Then
        raw_emit "  DriveThrough"
    End If
    If Flipz Then
        raw_emit "  FlipZ"
    End If
    raw_emit "end"
End Sub

'' The teleporter (but this takes an optional symbolic name for easy linking).
Sub pTeleporter(x, y, z, rotation, sizeX, sizeY, sizeZ, sizeB, Optional tName As String)
'    NumTeleporters = NumTeleporters + 1
'    istart = (NumTeleporters - 1) * 2
'    iend = istart + 1
'    TeleporterName(NumTeleporters) = tName
'    TeleporterFront(NumTeleporters) = istart
'    TeleporterBack(NumTeleporters) = iend
    raw_emit "teleporter"
    raw_emit "  position " & nArg(x) & " " & nArg(y) & " " & nArg(z)
    raw_emit "  rotation " & nArg(rotation)
    raw_emit "  border " & nArg(sizeB)
    raw_emit "  size " & nArg(sizeX) & " " & nArg(sizeY) & " " & nArg(sizeZ)
    raw_emit "end"
End Sub

'' The link.
Sub makelink(startT, endT)
    NumLinks = NumLinks + 1
    emit "link"
    emit " from " & CStr(startT)
    emit " to " & CStr(endT)
    emit "end"
End Sub

Sub Literal_Insert(inFname As String)
    Dim FFile As Integer
    Dim inBuffer As String
    
    FFile = FreeFile
    Open inFname For Input As #FFile
    Do While Not EOF(FFile)
        Line Input #FFile, inBuffer
        raw_emit inBuffer
    Loop
    Close #FFile
End Sub

