Attribute VB_Name = "Highways"
Public Const Highway_Width = 15
Public Const Highway_Separation = 5
Public Const Highway_Thickness = 2


''''''''''''''''''''''''''''
''' Highway Construction '''
''''''''''''''''''''''''''''


Sub highway(z As Double, p1x As Double, p1y As Double, p2x As Double, p2y As Double)
    'Construct a highway between points (p1x, p1y) and (p2x, p2y)
    Dim deltaX As Double
    Dim deltaY As Double
    Dim CenterX
    Dim CenterY
    Dim angle As Double
    Dim Offset
    Dim length, HalfLength As Double
    Const DeltaMain = 70
    Const EndBoxSize = 20
    
    Dim b1x As Double, b1y As Double, b2x As Double, b2y As Double
    Dim e1x As Double, e1y As Double, e2x As Double, e2y As Double
    
   
    CenterX = (p1x + p2x) / 2#
    CenterY = (p1y + p2y) / 2#
    
    deltaX = p2x - p1x
    deltaY = p2y - p1y
    
    length = Sqr(deltaX * deltaX + deltaY * deltaY) - 2 * EndBoxSize  '-10 for gap
    HalfLength = length / 2#
    angle = my_atan(deltaY, deltaX)
    
    'Do the two halves at appropriate points.
    Offset = Highway_Width / 2# + (Highway_Separation / 2#)
    b1x = CenterX - Offset * sinD(angle)
    b1y = CenterY + Offset * cosD(angle)
    b2x = CenterX + Offset * sinD(angle)
    b2y = CenterY - Offset * cosD(angle)
    
'    Box b1x, b1y, 0, Angle, Halflength, Highway_Width / 2#, 2
'    Box b2x, b2y, 0, Angle, Halflength, Highway_Width / 2#, 2
    
    'Rebind the deltaX, deltaY to have magnitude DeltaMain
    deltaX = DeltaMain * cosD(angle)
    deltaY = DeltaMain * sinD(angle)
    
    half_highway b1x, b1y, z, angle, HalfLength, -deltaX, -deltaY
    half_highway b2x, b2y, z, angle, HalfLength, deltaX, deltaY

    'boxes at ends
    MeshBox p1x, p1y, z, angle, EndBoxSize, EndBoxSize, Highway_Thickness
    MeshBox p2x, p2y, z, angle, EndBoxSize, EndBoxSize, Highway_Thickness
    
    
End Sub

Sub MeshBox(x As Double, y As Double, z As Double, angle As Double, sX As Double, sY As Double, sZ As Double, Optional inMaterial As String = "highway2")
    If Not V2 Then
        Box x, y, z, angle, sX, sY, sZ
        Exit Sub
    End If
    raw_emit ("meshbox")
    raw_emit ("  shift " & CStr(CInt(x)) & " " & CStr(CInt(y)) & " " & CStr(CInt(z)))
    'raw_emit ("  phydrv moving_sidewalk" & CStr(n))
    'raw_emit ("  size 14 500 0.9")
    raw_emit ("  size " & nArg(sX) & " " & nArg(sY) & " " & nArg(sZ))
    raw_emit ("  rotation " & nArg(angle + 90))
    raw_emit ("  angle 360")
    raw_emit ("  ratio 1")
    raw_emit ("  divisions 4")
    raw_emit ("  spin 0 0 0 0") 'last digit was 1
    raw_emit ("  shift 0 0 0")
    raw_emit ("  flatshading")
    raw_emit ("  texsize -8 -8 -8 -8")
    raw_emit ("  matref " & inMaterial)
    'raw_emit ("  bottom matref highway1")
    'raw_emit ("  inside matref highway1")
    'raw_emit ("  outside matref highway1")
    'raw_emit ("  startside matref highway1")
    'raw_emit ("  endside matref highway1")
    raw_emit ("end")

End Sub

Sub ColorMeshBox(x As Double, y As Double, z As Double, angle As Double, sX As Double, sY As Double, sZ As Double, inColor As String, Optional TopMaterial = "roof")
    If Not V2 Then
        Box x, y, z, angle, sX, sY, sZ
        Exit Sub
    End If
    raw_emit ("meshbox")
    raw_emit ("  shift " & CStr(CInt(x)) & " " & CStr(CInt(y)) & " " & CStr(CInt(z)))
    'raw_emit ("  phydrv moving_sidewalk" & CStr(n))
    'raw_emit ("  size 14 500 0.9")
    raw_emit ("  size " & nArg(sX) & " " & nArg(sY) & " " & nArg(sZ))
    raw_emit ("  rotation " & nArg(angle + 90))
'    raw_emit ("  angle 360")
'    raw_emit ("  ratio 1")
'    raw_emit ("  divisions 4")
'    raw_emit ("  spin 0 0 0 0") 'last digit was 1
'    raw_emit ("  shift 0 0 0")
'    raw_emit ("  flatshading")
    'raw_emit ("  texsize -8 -8 -8 -8")
    raw_emit ("  color " & inColor)
    'raw_emit ("  top matref " & TopMaterial)
    'raw_emit ("  bottom matref highway1")
    'raw_emit ("  inside matref highway1")
    'raw_emit ("  outside matref highway1")
    'raw_emit ("  startside matref highway1")
    'raw_emit ("  endside matref highway1")
    raw_emit ("end")

End Sub




Sub half_highway(x As Double, y As Double, z As Double, _
        angle As Double, HalfLength As Double, _
            dx As Double, dy As Double)
    
    Dim AllSize
    Dim n As Integer
    
    Highway_Number = Highway_Number + 1
    n = Highway_Number
    
    If Not V2 Then
        Box x, y, z, angle + 90, Highway_Width / 2#, HalfLength, Highway_Thickness
        Exit Sub
    End If
    
    'Create highway physics for this section
    highway_physics n, dx, dy
    
    raw_emit ("meshbox")
    raw_emit ("  shift " & nArg(x) & " " & nArg(y) & " " & nArg(z))
    raw_emit ("  phydrv moving_sidewalk" & CStr(n))
    'raw_emit ("  size 14 500 0.9")
    raw_emit ("  size " & nArg(Highway_Width / 2#) & " " & nArg(HalfLength) & " " & nArg(Highway_Thickness))
    raw_emit ("  rotation " & nArg(angle + 90))
    raw_emit ("  angle 360")
    raw_emit ("  ratio 1")
    raw_emit ("  divisions 4")
    raw_emit ("  spin 0 0 0 0") 'last digit was 1
    raw_emit ("  shift 0 0 0")
    raw_emit ("  flatshading")
    raw_emit ("  texsize -8 -8 -8 -8")
    raw_emit ("  matref highway1")
    'raw_emit ("  bottom matref highway1")
    'raw_emit ("  inside matref highway1")
    'raw_emit ("  outside matref highway1")
    'raw_emit ("  startside matref highway1")
    'raw_emit ("  endside matref highway1")
    raw_emit ("end")
End Sub

Sub highway_physics(n As Integer, dx, dy)
    'Set up physics
    raw_emit ("physics")
    raw_emit ("  name moving_sidewalk" & CStr(n))
    raw_emit ("  linear " & nArg(dx) & " " & nArg(dy) & " 0")
    raw_emit ("end")
    
End Sub

Sub highway_material()
    If Not V2 Then Exit Sub
    highway_matrix
    raw_emit ("material")
    raw_emit ("  name highway1")
'    raw_emit ("  diffuse 1 0 0 1")
'    raw_emit ("  specular 0 0 0 0")
'    raw_emit ("  emission 0 0 0 0")
    'raw_emit ("  addtexture green_bolt")
    raw_emit ("texture wall")
    raw_emit ("  texmat highwaym")
    raw_emit ("  notexalpha")
    raw_emit ("end")

    raw_emit ("material")
    raw_emit ("  name highway2")
'    raw_emit ("  diffuse 1 0 0 1")
'    raw_emit ("  specular 0 0 0 0")
'    raw_emit ("  emission 0 0 0 0")
    'raw_emit ("  addtexture green_bolt")
    raw_emit ("texture wall")
    'raw_emit ("  texmat highwaym")
    raw_emit ("  notexalpha")
    raw_emit ("end")
    

End Sub

Sub highway_matrix()
    raw_emit ("textureMatrix")
    raw_emit ("  name highwaym")
    raw_emit ("  shift 1 1")
    raw_emit ("end")
End Sub


