Attribute VB_Name = "Hills"
'''''''''''''
''' Hills '''
'''''''''''''

Sub test_hill(x As Double, y As Double, z As Double)
    Const xSize = 20
    Const ySize = 30
    Const zSize = 40
    
    If Not V2 Then Exit Sub
    
    Box x, y, 0, 0, xSize, ySize, 4 'just to delimit the platform.
    Box x, y, 20, 0, 1, 1, 30 'Pointer
    
    raw_emit ("material")
    raw_emit ("  name grass")
    raw_emit ("  texture std_ground")
    raw_emit ("  notexalpha")
    raw_emit ("end")
    
    raw_emit "mesh"
    raw_emit "  shift " & CStr(CInt(x)) & " " & CStr(CInt(y)) & " 0"
    raw_emit "  inside 0 0 10"
    raw_emit "  matref grass"
    
    'Do vertices
    vertex -xSize, ySize, 0
    vertex xSize, ySize, 0
    vertex -xSize, -ySize, 0
    vertex xSize, -ySize, 0
    vertex 0, 0, zSize
    
    'Do faces
    face3 2, 4, 0 '0, 4, 2
    face3 3, 4, 2 '2, 4, 3
    face3 1, 4, 3 '3, 4, 1
    face3 4, 1, 0 ' 0, 1, 4
    
    raw_emit "end"
    End
End Sub

Sub vertex(x, y, z)
    raw_emit "  vertex " & nArg(x) & " " & nArg(y) & " " & nArg(z)
End Sub

Sub face3(v1, V2, v3, Optional DriveThrough = False)
    raw_emit "    face"
    raw_emit "       vertices " & vArg(v1) & " " & vArg(V2) & " " & vArg(v3)
    If DriveThrough Then
        raw_emit "   drivethrough"
    End If
    raw_emit "    endface"
End Sub

Sub face4(v1, V2, v3, v4, Optional DriveThrough = False)
    raw_emit "    face"
    raw_emit "       vertices " & vArg(v1) & " " & vArg(V2) & " " & vArg(v3) & " " & vArg(v4)
    If DriveThrough Then
        raw_emit "   drivethrough"
    End If
    raw_emit "    endface"
End Sub

Sub hill_material()
    '' Material spec for grassy hills (move out after testing)
    If Not V2 Then Exit Sub
    raw_emit ("material")
    raw_emit ("  name grass")
    raw_emit ("  texture std_ground")
    raw_emit ("  notexalpha")
    raw_emit ("end")
End Sub

Sub hill(rFactor As Double, x As Double, y As Double, xLength As Double, yLength As Double, _
         xSteps As Integer, ysteps As Integer, Height As Double, Optional steep As Boolean = False)
    'Iteration variables
    Dim i As Integer
    Dim j As Integer
    Dim X_Stepsize As Double
    Dim Y_Stepsize As Double
    
    'Vertex coordinates
    Dim xcoord As Double
    Dim ycoord As Double
    Dim zcoord As Double
    
    'Vertex ID's
    Dim VertexIDs() As Integer  'The set of vertices
    Dim iVertexID As Integer        'The current vertex
    
    Dim Vert1 As Integer
    Dim Vert2 As Integer
    Dim Vert3 As Integer
    Dim Vert4 As Integer
    
    Dim tString As String
    
    If Not V2 Then Exit Sub
    
    ReDim VertexIDs(xSteps, ysteps)
    
    '' Marker boxes, for debugging
'    Box x, y, 0, 0, 1, 1, 30 'Overhead center
'    Box x, y, 3, 0, xLength / 2#, yLength / 2#, 1
        
    'Marker boxes for corners
    do_marker x - xLength / 2#, y - yLength / 2#
    do_marker x + xLength / 2#, y - yLength / 2#
    do_marker x + xLength / 2#, y + yLength / 2#
    do_marker x - xLength / 2#, y + yLength / 2#
    
    
    '' Emit the mesh header material
    raw_emit "mesh"
    raw_emit "  shift " & nArg(x) & " " & nArg(y) & " 0"
    raw_emit "  inside 10 10 10"
    raw_emit "  matref grass"
    
    '' First pass - build the vertices
    iVertexID = 0
    X_Stepsize = xLength / xSteps
    Y_Stepsize = yLength / ysteps
    
    Set_Random_Factor (rFactor)
    
    '' Emit vertices
    For i = 0 To xSteps
        For j = 0 To ysteps
        
            xcoord = i * X_Stepsize - (xLength / 2#) 'Subtraction to center the region.
            ycoord = j * Y_Stepsize - (yLength / 2#)
            
            'Compute Z value
            ''  If at edges, be at zero.
            If i = 0 Or i = xSteps Or j = 0 Or j = ysteps Then
                zcoord = 0#
            Else
                zcoord = Get_HillHeight(Height, i, j, xSteps, ysteps, steep)
            End If
            tString = "   vertex " & nArg(xcoord) & " " & nArg(ycoord) & " " & nArg(zcoord)
            tString = tString & " # Vertex ID: " & CStr(iVertexID)
            raw_emit tString
            VertexIDs(i, j) = iVertexID
            Debug.Print "i: " & CStr(i) & "(x=" & nArg(xcoord) & ", j: " & nArg(j) & " (y=" & nArg(ycoord) & "), VertexID: " & CStr(iVertexID)
            iVertexID = iVertexID + 1
        Next j
    Next i
    
    '' Now, emit faces.
    For i = 0 To xSteps - 1
        For j = 0 To ysteps - 1
            'Bind vertices
            Vert1 = VertexIDs(i, j)
            Vert2 = VertexIDs(i + 1, j)
            Vert3 = VertexIDs(i + 1, j + 1)
            Vert4 = VertexIDs(i, j + 1)
            raw_emit "  face"
            raw_emit "    vertices " & CStr(Vert1) & " " & CStr(Vert2) & " " & CStr(Vert4) 'Good
            raw_emit "  endface"
            raw_emit "  face"
            raw_emit "    vertices " & CStr(Vert2) & " " & CStr(Vert3) & " " & CStr(Vert4)
            raw_emit "  endface"
        Next j
    Next i
    
    raw_emit "end"
End Sub


Function Get_HillHeight(MaxHeight As Double, i As Integer, _
    j As Integer, xSteps As Integer, ysteps As Integer, Optional steep As Boolean = False)
    'Center
    Dim CenterX As Double
    Dim CenterY As Double
    Dim deltaX As Double
    Dim deltaY As Double
    Dim dMax As Double
    Dim DistX As Double
    Dim DistY As Double
    Dim Dist As Double
    Dim Ratio As Double
    
    Const DropOffPercent As Double = 0.3
    Dim DropOffSteps As Integer
    Dim xDsteps As Integer
    Dim yDsteps As Integer
    Dim xCentralMin As Integer
    Dim xCentralMax As Integer
    Dim yCentralMin As Integer
    Dim yCentralMax As Integer
    
    If Not steep Then
        CenterX = xSteps / 2#
        CenterY = ysteps / 2#
        dMax = Sqr((xSteps / 2#) * (xSteps / 2#) + (ysteps / 2#) * (ysteps / 2#))
        DistX = (i - CenterX)
        DistY = (j - CenterY)
        Dist = Sqr(DistX * DistX + DistY * DistY)
        'Dist will range from 0 at center to dMax
        Ratio = Dist / dMax
        
        If Dist = 0 Then 'we're at the center
            Get_HillHeight = MoreOrLess(MaxHeight)
        Else
            Get_HillHeight = MoreOrLess(MaxHeight * RGaussian(Ratio))
        End If
    Else
        'Want dropoff only in the outer 10% or so.
        '  For example: xsteps 100 (50 on each side of center)
        '  DropOffPercent = .10 (10%); dropoff should be 5 steps on each end.
        xDsteps = DropOffPercent * (xSteps / 2#)
        yDsteps = DropOffPercent * (ysteps / 2#)
        
        xCentralMin = xDsteps
        xCentralMax = xSteps - xDsteps
        yCentralMin = yDsteps
        yCentralMax = ysteps - yDsteps
        
        'Are we in the central area?
        If i > xCentralMin And i < xCentralMax And _
           j > yCentralMin And j < yCentralMax Then
           
            'We're in the normal, central region.
            Get_HillHeight = MoreOrLess(MaxHeight)
        Else
            Ratio = 1#
            If i <= xCentralMin Then
                'We have small x, figure ratio
                Ratio = i / xCentralMin
            Else
                If i >= xCentralMax Then
                    'big i, at upper end of scale
                    Ratio = (xSteps - i) / xDsteps
                End If
            End If
            
            'Now factor in y
            If j <= yCentralMin Then
                Ratio = Ratio * (j / yDsteps)
            Else
                If j >= yCentralMax Then
                    Ratio = Ratio * (ysteps - j) / yDsteps
                End If
            End If
            If Ratio > 1 Then
                MsgBox "odd ratio"
            End If
            'Get_HillHeight = MoreOrLess(MaxHeight * RGaussian(Ratio))
            Get_HillHeight = MoreOrLess(MaxHeight * Ratio)
        End If
    End If
    
End Function

Function RGaussian(inRatio As Double)
    'be near 1 for inRatio = 0
    'be near 0 for inratio = 1
    
    Dim Exponent As Double
    Exponent = 5# * inRatio * inRatio
    
    RGaussian = Exp(-Exponent)
    
End Function


Sub do_marker(x, y)
    If Not V2 Then Exit Sub
    MeshBox CDbl(x), CDbl(y), 0, 0, 1, 1, 5
End Sub


Sub Set_Random_Factor(inseed As Double)
    'reseed the random number generator so that we have
    'a repeatable random sequence.
    Rnd -1
    Randomize inseed
    
End Sub

Function MoreOrLess(GivenHeight) As Double
    Dim Delta
    Delta = Rnd() * 0.1 * GivenHeight
    If Rnd() > 0.5 Then
        Delta = -Delta
    End If
    MoreOrLess = GivenHeight + Delta
End Function
Sub testHhill(x, y, z)
    
    If Not V2 Then Exit Sub
    
    'The material
    raw_emit ("material")
    raw_emit ("  name grass")
    raw_emit ("  texture std_ground")
    raw_emit ("  notexalpha")
    raw_emit ("end")
        
    'The bounding box for quick reference
    
    'The mesh
    raw_emit "mesh"
    raw_emit "  matref grass"
    raw_emit "  shift " & CStr(x) & " " & CStr(y) & " " & CStr(z)
    raw_emit "  inside 0 0 7"
    raw_emit "  vertex 0 0 0"
    raw_emit "  vertex 0 100 0"
    raw_emit "  vertex 100 0 0"
    raw_emit "  vertex 0 0 100"
    raw_emit "  face"
    raw_emit "    vertices 0 1 2"
    raw_emit "  endface"
    raw_emit "  face"
    raw_emit "    vertices 3 2 1"
    raw_emit "  endface"
    raw_emit "  face"
    raw_emit "    vertices 3 1 0"
    raw_emit "  endface"
    raw_emit "  face"
    raw_emit "    vertices 3 0 2"
    raw_emit "  endface"
    raw_emit "end"
    
End Sub



'''''''''''''''''
''' Utilities '''
'''''''''''''''''

Function my_atan(dy, dx) As Double
    Dim raw As Double
    If dx = 0 Then
        'vertical line
        If dy > 0 Then
            my_atan = 90
        Else
            my_atan = -90
        End If
    Else
        my_atan = Atn(dy / dx) * 360# / (2# * 3.14159)
    End If
End Function



