Attribute VB_Name = "Machinery"
''''''''''''''''''''''''
''''''''''''''''''''''''                Bryan A. Bentz
''' MapGen Machinery '''                September 26, 2003
''''''''''''''''''''''''
''''''''''''''''''''''''


'''''''''''''''''''''''''''''''
''' Variables and Constants '''
'''''''''''''''''''''''''''''''


''' Variables used in transform computations
''     Doing a coordinate transform involving a rotation involves
''     simultaneously computing the resulting X and Y.  As there is
''     no VB mechanism for multiple return values, these variables
''     are used to hold the result.
Dim xform_x
Dim xform_y
Dim xform_theta
Dim xform_newX, xform_newY

Public ReflectX As Boolean
Public ReflectY As Boolean


''' Variables used in Walls
''     Crenelations and towers are added, and sized based on a percentage
''     of the specified height.  Sometimes it is useful for the caller
''     to have access to these heights; these variables are used for
''     passing back this data.
Public AddedTowerHeight     'The height that was used for towers.
Const CrenWidth = 0.3       'The thickness of the crenelations
Const CrenHeight = 1.6      'The height of crenelations - fixed to be
                            'just above the gunshot height.
Public Const default_bsize = 1
Public Decay_Wall_count

''' Teleporter support
''     It is often hard to keep track of teleporter IDs for assigning
''     links.  These variables help associate symbolic names to the
''     front and back of teleporters.
Const MaxTeleporters = 200
Public NumTeleporters As Integer
Public TeleporterName(MaxTeleporters) As String
Public TeleporterFront(MaxTeleporters) As Integer
Public TeleporterBack(MaxTeleporters) As Integer

Public NumBoxes As Long
Public NumPyramids As Long
Public NumTeleporterPlatforms As Integer
Public NumComments As Integer
Public NumElse As Integer
Public NumDirect As Integer
Public NumLinks As Integer
Public Const tFront As Integer = 1
Public Const tBack As Integer = 2

''' Generally useful variables
Const pi = 3.14159265358979

Public WorldOptions(100) As String
Public WorldValues(100) As String 'handle as literal strings, no pretty-printing.
Public NumWorldOptions As Integer



Public CurrentQueue As Integer
Public GQueue(20) As Queue

'' Queue 0 is the one which will be written out to file.

Sub initialize_queues()
    For i = 0 To 20
        Set GQueue(CurrentQueue) = New Queue
    Next i
    NumDirect = 0
End Sub

''' World Options
Sub WorldSet(variable As String, value As String)
    If ValidOption(variable) Then
        WorldOptions(NumWorldOptions) = variable
        WorldValues(NumWorldOptions) = value
        NumWorldOptions = NumWorldOptions + 1
    Else
        Error 1
    End If
End Sub

Function ValidOption(inOpt As String) As Boolean
    Dim i As Integer
    Const refstring = " _ColdetDepth _explodeTime _mirror _noClimb " & _
        " _noShadows " & "_rainType _syncLocation _syncTime _tankSpeed " & _
        " _trackFade _wallheight _wingsjumpcount _wingsslidetime " & _
        " _wingsjumpvelocity _burrowAngularAd _burrowSpeedAd _noRadar"
    i = InStr(1, LCase(refstring), LCase(inOpt))
    If i > 0 Then
        ValidOption = True
    Else
        ValidOption = False
    End If
End Function



'''''''''''''''''''''''
''' Derived objects '''
'''''''''''''''''''''''

'' Derived objects are the heart of the code: they are more complex
'' structures than the basic BZFlag elements.
''
'' It would be possible to package each of these up as first-class
'' objects, and provide methods to attach them to each other, or to
'' return critical information about the composite structures (cumulative
'' width or height, for example).  I've made an engineering decision that
'' this is overkill - at least for now.


''' Bar
''     This is a connection between two X Y points.

Sub bar(x1, y1, z1, x2, y2, z2, sizeX, sizeY, nsteps)
    Dim i As Integer
    Dim hsizeX
    Dim hsizeY
    Dim sizeZ
    Dim theta
    Dim xc, yc, Zc, dfraction
    hsizeX = sizeX / 2#
    hsizeY = sizeY / 2#
    sizeZ = (z2 - z1) / nsteps
    theta = my_find_angle(x2 - x1, y2 - y1)
    For i = 1 To nsteps
        dfraction = i / nsteps
        xc = x1 + (x2 - x1) * dfraction
        yc = y1 + (y2 - y1) * dfraction
        Zc = z1 + (z2 - z1) * dfraction
'        Box xc, yc, zc, 90 - theta, hsizeX, hsizeY, sizeX
        Box xc, yc, Zc, theta, hsizeX, hsizeY, sizeX
    
    Next i
End Sub


''' Cylinder
''     For debugging, right now this is just a box.

Sub cylinder(x, y, z, Totallength, radius, theta)
    'Box x, y, z, rot, sx, sy, sz
    Box x, y, z, theta, radius / 2, Totallength / 2, radius
End Sub


''' Curved Walls
''     Curved walls are defined by an (X, Y) center point, a radius,
''     and start and end angles.  Another parameter is which side the
''     crenelations are on, which of course depends on whether it is
''     intended as a concave or convex defensive wall.
''
''     Other arguments are the wall width, the number of segments used
''     in the polygon approximation, the total number of towers, whether
''     crenelations are desired, if so how long the open and closed parts
''     of the crenelation are, and "crenelout", how much to outdent the
''     crenelations.
''
''     (This is one of the oldest functions here, and initially I had
''      a different orientation of X and Y, and a different means of
''      doing transforms.  So the code isn't the cleanest, but it works.)

Sub circular_wall(FirstArgX, SecondArgY, startz, radius, thetaStart, thetaEnd, Height, _
        width, _
      numsegments, numtowers, crenelation As Boolean, crenopenwidth, crenclosedwidth, _
      crenside, Optional crenelout = 0)
   Dim origFullRadius
   Dim orginX, originY
   Dim i As Integer
   Dim thetaSection As Double
   Dim radiansStart
   Dim radiansEnd
   Dim angle_start
   Dim angle_end
   Dim angle_center
   Dim boxX
   Dim boxY
   Dim boxangle
   Dim bLength
   Dim distance, twidth, theight, trot
   originX = SecondArgY
   originY = FirstArgX
   comment "### Circular wall section."
   thetaSection = (thetaEnd - thetaStart) / numsegments
 '  bLength = (radius + width/2) * Abs(sinD(thetaSection)) / 2#
   bLength = (radius + width) * Abs(sinD(thetaSection)) / 2#
   'stretch a little
   bLength = bLength * 1.1
   
   For i = 0 To numsegments
        angle_start = (i - 1) * thetaSection + thetaStart
        angle_end = i * thetaSection + thetaStart
        angle_center = (angle_end + angle_start) / 2# ''Average
        boxX = radius * sinD(angle_center) + originX
        boxY = radius * cosD(angle_center) + originY
        boxangle = angle_center ''Angles are in degrees.
        Box boxY, boxX, startz, boxangle, width, bLength, Height
    Next i
    
    If numtowers = 0 Then GoTo crenels
    comment "### Towers"
    thetaSection = (thetaEnd - thetaStart) / numtowers
    twidth = width * 4
    theight = Height * 1.3
    AddedTowerHeight = theight  'So caller can match this
    For i = 1 To numtowers
        angle_center = (CDbl(i) - 0.5) * thetaSection + thetaStart
        boxX = radius * sinD(angle_center) + originX
        boxY = radius * cosD(angle_center) + originY
        boxangle = angle_center
        tower boxY, boxX, startz, boxangle, twidth, theight, crenelation
    Next i
    
crenels:
    If Not crenelation Then Exit Sub
    comment "### Crenelations."
    'Insert crenelations!
    origFullRadius = radius + width + crenelout
    distance = ((thetaEnd - thetaStart) / 360#) * pi * 2# * origFullRadius
    numsegments = distance / (crenopenwidth + crenclosedwidth)
    bLength = crenclosedwidth
    thetaSection = (thetaEnd - thetaStart) / numsegments
     For i = 0 To numsegments
        angle_start = (i - 1) * thetaSection + thetaStart
        angle_end = i * thetaSection + thetaStart
        angle_center = (angle_end + angle_start) / 2# ''Average
        If crenside = 1 Then 'outside of circular curve
            boxX = (origFullRadius - CrenWidth + crenelout) * sinD(angle_center) + originX
            boxY = (origFullRadius - CrenWidth + crenelout) * cosD(angle_center) + originY
        Else
            origFullRadius = radius - width
            boxX = (origFullRadius - CrenWidth + crenelout) * sinD(angle_center) + originX
            boxY = (origFullRadius - CrenWidth + crenelout) * cosD(angle_center) + originY
        End If
        boxangle = angle_center ''Angles are in degrees.
        'Box boxX, boxY, 0, boxangle, width, blength, height / 2#
        Box boxY, boxX, startz + Height, boxangle, CrenWidth, bLength, CrenHeight
    Next i
  
End Sub


''' Straight Walls
''     Similar to Circular Walls in terms of arguments.  Note that this
''     call takes a starting point (X1, Y1) and an ending point (X2, Y2),
''     not an angle.  I've found this much more convenient when trying to
''     put wall fragments together.

Sub straight_wall(x1, y1, x2, y2, startz, _
        width, Height, numtowers, crenelation As Boolean, _
        crenopenwidth, crenclosedwidth, crenside, Optional crenelout = 0)
    
    Dim xcenter, ycenter, xdelta, ydelta, wallLength, theta
    Dim numsegments, crenX, crenY, xside, yside, newX1, newY1
    Dim twidth, theight
    Dim i As Integer
    
    xcenter = (x1 + x2) / 2#
    ycenter = (y1 + y2) / 2#
    xdelta = x2 - x1
    ydelta = y2 - y1
    wallLength = Sqr(xdelta * xdelta + ydelta * ydelta)
    theta = my_find_angle(xdelta, ydelta)
    Box xcenter, ycenter, startz, theta, wallLength / 2#, width, Height
    
    'Do towers
    twidth = width * 4
    theight = Height * 1.3
    AddedTowerHeight = theight
    For i = 1 To numtowers
        newX1 = x1 + i * (xdelta / numtowers)
        newY1 = y1 + i * (ydelta / numtowers)
        tower newX1, newY1, startz, theta, twidth, theight, crenelation
    Next i
    
    If crenelation Then
        'Do crenelations
        If crenside = 1 Then
            newX1 = x1 + width * sinD(theta)
            newY1 = y1 - width * cosD(theta)
        Else
            newX1 = x1 - width * sinD(theta)
            newY1 = y1 + width * cosD(theta)
        End If
        numsegments = wallLength / (crenopenwidth + crenclosedwidth)
        For i = 1 To numsegments - 1
                crenX = newX1 + i * (xdelta / numsegments)
                crenY = newY1 + i * (ydelta / numsegments)
            Box crenX, crenY, startz + Height, theta, crenclosedwidth, CrenWidth, CrenHeight
        Next i
    End If
End Sub


Sub decaying_wall(x1, y1, x2, y2, startz, _
        width, theight)
        
    ' Construct a wall with nearby fragments that gradually
    ' diminishes from (x1, y1) to (x2, y2)
        
    Dim xcenter, ycenter, xdelta, ydelta, theta
    Dim numsegments As Integer
    Dim i As Integer
    Dim xBlock As Double, yBlock As Double
    Dim ThisHeight As Double
    Dim RemnantHeight As Double, RemnantY As Double
    Dim Sname As String
    Dim Xstart As Double, ystart As Double
    Dim xSize As Double, ySize As Double
    Dim wallLength As Double, HalfWallLength As Double
    Dim Height As Double
    Dim Blocksize As Double
    Const SpreadDist = 6
    
    Height = 0.95 * theight
    Blocksize = 2 * width
    xSize = x2 - x1
    ySize = y2 - y1
    wallLength = Sqr(xSize * xSize + ySize * ySize)
    HalfWallLength = wallLength / 2#
    numsegments = wallLength / Blocksize '10 'Number of verticals we examine.
    
    Sname = "Decaying Wall " & CStr(Decay_Wall_count)
    Decay_Wall_count = Decay_Wall_count + 1
    
    define_component Sname
        'Do this centrally around the origin
        For i = 1 To numsegments
            xBlock = (i * wallLength / numsegments) - HalfWallLength
            yBlock = 0
            ThisHeight = Height - i * (Height / numsegments)
            ThisHeight = MoreOrLess(ThisHeight)
            'Remaining wall segment
            Box xBlock, yBlock, 0, 0, wallLength / numsegments, width, ThisHeight
            'Do remnant on ground
            RemnantY = yBlock + 2 * width + SpreadDist * Rnd()
            If Rnd() > 0.5 Then
                RemnantY = -RemnantY
            End If
            'Fragment lying down next to other block
            'Box xBlock, RemnantY, 0, 180# * Rnd(), Height - ThisHeight, WallLength / numsegments, width
            If Rnd() > 0.7 Then
                Box xBlock, RemnantY, 0, 180# * Rnd(), width, wallLength / numsegments, width
            End If
        Next i
    end_component
    
    xcenter = (x1 + x2) / 2#
    ycenter = (y1 + y2) / 2#
    xdelta = x2 - x1
    ydelta = y2 - y1
    wallLength = Sqr(xdelta * xdelta + ydelta * ydelta)
    theta = my_find_angle(xdelta, ydelta)
    
    'Was a single box
    'Box xcenter, ycenter, startz, theta, wallLength / 2#, width, Height
    Place_Component Sname, xcenter, ycenter, startz, theta
End Sub



Sub BoxConnect(x1, y1, x2, y2, z, width, Height)
    
    Dim xcenter, ycenter, xdelta, ydelta, wallLength, theta
    Dim twidth, theight
    Dim i As Integer
    
    xcenter = (x1 + x2) / 2#
    ycenter = (y1 + y2) / 2#
    xdelta = x2 - x1
    ydelta = y2 - y1
    wallLength = Sqr(xdelta * xdelta + ydelta * ydelta)
    theta = my_find_angle(xdelta, ydelta)
    Box xcenter, ycenter, z, theta, wallLength / 2#, width, Height
End Sub


''' Towers
''     This function is used as an internal in the wall code above, but
''     is also useful to have generally available.

Sub tower(y, x, startz, theta, width, Height, Optional crenels As Boolean = True)
    Dim crenSpace
    Dim i As Integer
    Dim j As Integer
    Dim crenX, crenY
    Dim xmin, xmax, ymin, ymax, xstep, ystep
    Dim newX, newY
    'Const CrenWidth = 0.3  'These may be set here to override the defaults.
    'Const CrenHeight = 1.6
    Dim r, angle, newangle
    Dim oldX, oldY, halfdelta
    Const NumCrens = 5
    crenSpace = (2 * width) / (4 * NumCrens)
    Box y, x, startz, theta, width, width, Height
    
    If Not crenels Then Exit Sub
    'Now crenelate it.
    
    'Run from x=-halfwidth to x=+halfwidth
    'Run from y =-halfwidth to y=+halfwidth
    '(one coord always pinned to max)
    
    'To compute new coords xprime
    'xprime=x*cos(theta)+offset
    'yprime=y*cos(theta)+offset
    xmin = -width
    xmax = width
    ymin = -width
    ymax = width
    xdelta = 2 * width / NumCrens
    ydelta = 2 * width / NumCrens
    halfdelta = xdelta / 2#
    For i = 0 To NumCrens - 1
            ' For debugging, put in a comment in the generated file.
            'comment "# i=" & CStr(i)
            oldX = xmax - i * xdelta - halfdelta
            oldY = ymax
            'do transform and draw
            Call set_transform_values(oldX, oldY, theta)
            newX = transform_result_x() + x
            newY = transform_result_y() + y
            Box newY, newX, startz + Height, theta, CrenWidth, crenSpace, CrenHeight
            
            oldX = xmax - i * xdelta - halfdelta
            oldY = ymin
            'do transform and draw
            Call set_transform_values(oldX, oldY, theta)
            newX = transform_result_x() + x
            newY = transform_result_y() + y
            Box newY, newX, startz + Height, theta, CrenWidth, crenSpace, CrenHeight
'
            oldX = xmax
            oldY = ymax - i * ydelta - halfdelta
            'do transform and draw
            Call set_transform_values(oldX, oldY, theta)
            newX = transform_result_x() + x
            newY = transform_result_y() + y
            Box newY, newX, startz + Height, theta + 90, CrenWidth, crenSpace, CrenHeight
'
            oldX = xmin
            oldY = ymax - i * ydelta - halfdelta
            'do transform and draw
            Call set_transform_values(oldX, oldY, theta)
            newX = transform_result_x() + x
            newY = transform_result_y() + y
            Box newY, newX, startz + Height, theta + 90, CrenWidth, crenSpace, CrenHeight
    Next i
End Sub


''' Arches
''     Someone will probably complain about the number of polygons that
''     these require, but they do add a neat touch.  However, I'd recommend
''     going lightly with them - as tempting as it might be to construct
''     an aqueduct with 50 or so, it might have a significant performance
''     impact.
''
''     GateHeight is the height of the rectangular part of the gateway,
''     above which is constructed the semicircular arch.  You could
''     probably set it to zero with a big enough GateWidth and still
''     be able to get through.  Thickness is how long the arch is, in
''     the direction you drive through it.  Border is how wide it is
''     on either side of the roadway as you enter the arch.

Sub arch(xc, yc, Zc, theta, GateHeight, gateWidth, thickness, Border, Optional inLite As Boolean = False)
    Dim fullWidth, fullHeight, x1, x2, y1, y2
    Dim leftpostx, rightpostx, leftposty, rightposty
    Dim nX, nY, nZ, vbarheight, rx, ry
    Const numSteps = 30
    Dim StepAngle, StepX, StepY, StepRadius, stepXwidth, maxX, totalWidth, halfwidth
    Dim ArchMaxZ
    Dim TopBarTotalWidth, TopBarHalfWidth, TopBarBottom, TopBarZ
    Dim i As Integer
    
    leftpostx = -(gateWidth / 2#) - Border / 2#
    leftposty = 0
    rightpostx = -leftpostx
    rightposty = 0
    
    If Not inLite Then
        ''Angle fill-in bars.
        ArchMaxZ = 0
        StepAngle = 100 / numSteps
        StepRadius = gateWidth / 2#
        maxX = gateWidth / 2#
        For i = 0 To numSteps
            nX = StepRadius * cosD(i * StepAngle)
            totalWidth = maxX - nX
            halfwidth = totalWidth / 2#
            nX = nX + halfwidth
            nY = 0
            nZ = GateHeight + StepRadius * sinD(i * StepAngle)
            If nZ > ArchMaxZ Then ArchMaxZ = nZ
            stepXwidth = halfwidth 'StepRadius - nX
            Call set_transform_values(nX, nY, theta)
            rx = transform_result_x() + xc
            ry = transform_result_y() + yc
            If rx = 0 Or ry = 0 Then GoTo nexti
            Box rx, ry, nZ, -theta, stepXwidth, thickness / 2, Border / 5
            
            Call set_transform_values(-nX, nY, theta)
            rx = transform_result_x() + xc
            ry = transform_result_y() + yc
            Box rx, ry, nZ, -theta, stepXwidth, thickness / 2, Border / 5
nexti:
        Next i
    Else
        ArchMaxZ = GateHeight
    End If
    
    'Left gate post
    Call set_transform_values(leftpostx, leftposty, theta)
    nX = transform_result_x() + xc
    nY = transform_result_y() + yc
    vbarheight = ArchMaxZ ' gateHeight
    Box nX, nY, Zc, -theta, Border / 2, thickness / 2, vbarheight
    
    'right gate post
    Call set_transform_values(rightpostx, rightposty, theta)
    nX = transform_result_x() + xc
    nY = transform_result_y() + yc
    Box nX, nY, Zc, -theta, Border / 2, thickness / 2, vbarheight
    
    'top bar
    TopBarTotalWidth = gateWidth + 2 * Border
    TopBarHalfWidth = TopBarTotalWidth / 2#
    TopBarBottom = vbarheight
    topbarheight = Border
    TopBarZ = TopBarBottom '+ topbarheight / 2#
    Box xc, yc, Zc + TopBarZ, -theta, _
       TopBarHalfWidth, thickness / 2, Border / 2
End Sub

Sub MinArch(xc, yc, Zc, theta, GateHeight, gateWidth, thickness, Border, Optional inLite As Boolean = False)
    Dim fullWidth, fullHeight, x1, x2, y1, y2
    Dim leftpostx, rightpostx, leftposty, rightposty
    Dim nX, nY, nZ, vbarheight, rx, ry
    Const numSteps = 30
    Dim StepAngle, StepX, StepY, StepRadius, stepXwidth, maxX, totalWidth, halfwidth
    Dim ArchMaxZ
    Dim TopBarTotalWidth, TopBarHalfWidth, TopBarBottom, TopBarZ
    Dim i As Integer
    
    leftpostx = -(gateWidth / 2#) - Border / 2#
    leftposty = 0
    rightpostx = -leftpostx
    rightposty = 0
    
    If Not inLite Then
        ''Angle fill-in bars.
        ArchMaxZ = 0
        StepAngle = 100 / numSteps
        StepRadius = gateWidth / 2#
        maxX = gateWidth / 2#
        For i = 0 To numSteps
            nX = StepRadius * cosD(i * StepAngle)
            totalWidth = maxX - nX
            If totalWidth = 0 Then GoTo nexti
            halfwidth = totalWidth / 2#
            nX = nX + halfwidth
            nY = 0
            nZ = GateHeight + StepRadius * sinD(i * StepAngle)
            If nZ > ArchMaxZ Then ArchMaxZ = nZ
            stepXwidth = halfwidth 'StepRadius - nX
            Call set_transform_values(nX, nY, theta)
            rx = transform_result_x() + xc
            ry = transform_result_y() + yc
            'box x, y, z, theta, sizeX, sizeY, sizeZ
            Box rx, ry, nZ, -theta, stepXwidth, thickness / 2, Border / 5
            
            Call set_transform_values(-nX, nY, theta)
            rx = transform_result_x() + xc
            ry = transform_result_y() + yc
            Box rx, ry, nZ, -theta, stepXwidth, thickness / 2, Border / 5
nexti:
        Next i
    Else
        ArchMaxZ = GateHeight
    End If
getout:
    'top bar
'    TopBarTotalWidth = gateWidth + 2 * Border
'    TopBarHalfWidth = TopBarTotalWidth / 2#
'    TopBarBottom = vbarheight
'    topbarheight = Border
'    TopBarZ = TopBarBottom '+ topbarheight / 2#
'    Box xc, yc, Zc + TopBarZ, -theta, _
'       TopBarHalfWidth, thickness / 2, Border / 2
End Sub


''' Bordering around blocks
''     Sometimes for large blocks, it's nice to have a roadway going
''     around the block, e.g. as a ledge one can drive on, to provide
''     access to different heights.  Rather than having 4 different
''     rectangles for each of the sides, it is simpler to have one single
''     block which sticks out appropriately.

Sub Border_Block(x, y, z, width, length, theta, borderwidth, borderheight, Optional thickness = 5)
    'Box x, y, z, rot, sx, sy, sz
    Box x, y, z + borderheight, theta, width + borderwidth, length + borderwidth, thickness
End Sub


''' jfort_tower
''     "jfort" became the name of a style; it was called this because
''     in the first script I wrote, I prefixed all of the parameters
''     with "j".  It is a simple block with a pyramid on top (up a tiny
''     bit so you can get inside it).  It is constructed to reach down
''     below the z value given, so that the result looks something like
''     a turret.  Initially this was just in-line coded, but became common
''     enough to break out.

Sub jfort_tower(x, y, z)
    Const zdown = 4
    'box x, y, z, rot, sx, sy, sz
    Box x, y, z - zdown, 0, jtWidth, jtWidth, jtHeight
    'pyramid x, y, z, rot, sx, sy, sz
    Pyramid x, y, z - zdown + jtHeight, 0, _
        jtWidth, jtWidth, jtHeight / 4
End Sub

''' Teleporters
''     Teleporter platforms are floating strategic squares with
''     teleporters in the center.

Sub teleporter_platform(x, y, z, theta, Optional tName As String = "NONAME")
    Dim realname As String
    'NumTeleporterPlatforms starts at zero (when there are none).
    NumTeleporterPlatforms = NumTeleporterPlatforms + 1
    If tName = "NONAME" Then
        'The user didn't specify one; make one up.
        realname = "Teleporter Floating Platform " & CStr(NumTeleporterPlatforms)
    Else
        realname = tName
    End If
    Box x, y, z, theta, 30, 30, 5
    Teleporter x, y, z + 5, theta, 0.56, 10, 10, 1.12, realname
End Sub

Sub fully_connect_teleporters(t1name As String, t2name As String)
    comment "Fully connect " & t1name & " and " & t2name & " START"
    make_teleporter_link t1name, tFront, t2name, tFront
    make_teleporter_link t1name, tBack, t2name, tBack
    make_teleporter_link t2name, tFront, t1name, tFront
    make_teleporter_link t2name, tBack, t1name, tBack
    comment "Fully connect " & t1name & " and " & t2name & " END"
End Sub

Sub make_teleporter_link(t1name As String, t1side, t2name As String, t2side)
    
    'Should queue up a link, to process later!!!
    
    Dim Teleporter1 As Integer
    Dim Teleporter2 As Integer
    Dim id1 As Integer
    Dim id2 As Integer
    
    Debug.Print "Linking " & t1name & "(" & CStr(t1side) & ") to " & t2name & "(" & CStr(t2side) & ")"
    
    Teleporter1 = Find_Teleporter_ID(t1name)
    Teleporter2 = Find_Teleporter_ID(t2name)
    If Teleporter1 = 0 Then
        MsgBox "MAKE_TELEPORTER_LINK: Could not find teleporter named """ & t1name & """"
        Exit Sub
    End If
    If Teleporter2 = 0 Then
        MsgBox "MAKE_TELEPORTER_LINK: Could not find teleporter named """ & t2name & """"
        Exit Sub
    End If
    
    '' Figure out ID's based on side
    If t1side = tFront Then
        id1 = TeleporterFront(Teleporter1)
    Else
        id1 = TeleporterBack(Teleporter1)
    End If
    If t2side = tFront Then
        id2 = TeleporterFront(Teleporter2)
    Else
        id2 = TeleporterBack(Teleporter2)
    End If
'    If id1 = 40 Or id1 = 41 Then
'        MsgBox "Here"
'    End If
    'Emit this specific link.
    makelink id1, id2
End Sub

Function Find_Teleporter_ID(tName As String)
    Dim i As Integer
    iret = 0
    For i = 1 To MaxTeleporters
        If TeleporterName(i) = tName Then
            Find_Teleporter_ID = i
            Exit Function
        End If
    Next i
    Find_Teleporter_ID = 0
End Function

''''''''''''''''''''''''''''''''''
''' Random, scattered pyramids '''
''''''''''''''''''''''''''''''''''

Sub RoadPyramid(x, y, Optional inLite As Boolean = False)
    Dim xc, yc, pHeight, theta, xSize, ySize
    Const Delta = 70
    Const MaxHeight = 35
    Const maxsize = 10
    Const MinHeight = 1.6
    Dim pnum
    If inLite Then
        pnum = 8
    Else
        pnum = 20
    End If
    
    Dim anum
    Dim i As Integer
    anum = pnum * Rnd()
    For i = 1 To anum
        xc = x + Delta * Rnd()
        yc = y + Delta * Rnd()
        pHeight = MaxHeight * Rnd() + MinHeight
        xSize = maxsize * Rnd()
        ySize = maxsize * Rnd()
        theta = 360 * Rnd()
        Pyramid xc, yc, 0, theta, xSize, ySize, pHeight
    Next i
End Sub

Sub Scattered_Pyramids(x, y, Maxnum, deltaX, deltaY)
    Dim xc, yc, pHeight, theta, xSize, ySize
    Const MaxHeight = 35
    Const maxsize = 10
    Const MinHeight = 1.6
    Dim pnum
    pnum = Maxnum
    
    Dim anum
    Dim i As Integer
    anum = pnum * Rnd()
    For i = 1 To anum
        xc = x + deltaX * Rnd()
        yc = y + deltaY * Rnd()
        pHeight = MaxHeight * Rnd() + MinHeight
        xSize = maxsize * Rnd()
        ySize = maxsize * Rnd()
        theta = 360 * Rnd()
        If xc < 400 And xc > -400 And yc < 400 And yc > -400 Then
            Pyramid xc, yc, 0, theta, xSize, ySize, pHeight
        End If
    Next i
End Sub

Sub Uniform_Random_Pyramids(x, y, Maxnum, deltaX, deltaY, psize, pHeight)
    Dim xc, yc, theta, xSize, ySize
    Dim i As Integer
    Dim iSignX As Integer
    Dim iSignY As Integer
    For i = 1 To Maxnum
    If Rnd() > 0.5 Then
        iSignX = -1
    Else
        iSignX = 1
    End If
    If Rnd() > 0.5 Then
        iSignY = -1
    Else
        iSignY = 1
    End If
        xc = x + iSignX * deltaX * Rnd()
        yc = y + iSignY * deltaY * Rnd()
        xSize = psize
        ySize = psize
        theta = 0
        Pyramid xc, yc, 0, theta, xSize, ySize, pHeight
    Next i
End Sub

Sub Uniform_Region_Pyramids(minX, minY, maxX, maxY, density, psize, pHeight)
    Dim xc, yc, theta, xSize, ySize, deltaX, deltaY, x, y
    Dim i As Integer
    Dim iSignX As Integer
    Dim iSignY As Integer
    
    x = (minX + maxX) / 2#
    y = (minY + maxY) / 2#
    deltaX = maxX - xc
    deltaY = maxY - yc
    
    For i = 1 To Maxnum
        If Rnd() > 0.5 Then
            iSignX = -1
        Else
            iSignX = 1
        End If
        If Rnd() > 0.5 Then
            iSignY = -1
        Else
            iSignY = 1
        End If
        xc = x + iSignX * deltaX * Rnd()
        yc = y + iSignY * deltaY * Rnd()
        xSize = psize
        ySize = psize
        theta = 0
        Pyramid xc, yc, 0, theta, xSize, ySize, pHeight
    Next i
End Sub
'''''''''''''''''''''''''''''''
''' Trigonometric Functions '''
'''''''''''''''''''''''''''''''

'' While the underlying VB functions require radians, BZFlag uses
'' degrees as coordinates - these first two functions take degrees
'' and convert them to radians.

Function sinD(inNum) As Double
    Dim radians As Double
    radians = (pi * inNum) / 180#
    sinD = Sin(radians)
End Function

Function cosD(inNum) As Double
    Dim radians As Double
    radians = (pi * inNum) / 180#
    cosD = Cos(radians)
End Function


'' Often it is necessary to specify an angle to BZFlag, when
'' what the author wants to write down involves a starting (X, Y) point
'' and an ending (X, Y) point.  This function robustly computes the
'' equivalent angle.

Public Function my_find_angle(inX, inY) As Double
    Dim quadrant
    Dim angle
    If inX > 0 Then
        If inY > 0 Then
            quadrant = 1
        Else
            quadrant = 4
        End If
    Else
        If inY > 0 Then
            quadrant = 2
        Else
            quadrant = 3
        End If
    End If
    
    If inX = 0 Then
        'We are on the Y axis, either positive or negative
        If Sgn(inY) = 1 Then
            my_find_angle = 90
        Else
            my_find_angle = 270
        End If
        Exit Function
    End If
    
    angle = Atn(Abs(inY / inX)) * (180 / pi) 'Will be +90 to -90
    Select Case quadrant
        Case 1
            my_find_angle = angle
        Case 2
            my_find_angle = 180 - angle
        Case 3
            my_find_angle = angle + 180
        Case 4
            my_find_angle = 360 - angle
    End Select
    If my_find_angle = 360 Then
        my_find_angle = 0
    End If
End Function


'''''''''''''''''''''''
''' Transformations '''
'''''''''''''''''''''''

Sub set_transform_values(oldX, oldY, theta)
    Dim r, angle, newangle
    xform_x = oldX
    xform_y = oldY
    xform_theta = theta
    r = Sqr(oldX * oldX + oldY * oldY)
    angle = my_find_angle(oldX, oldY)
    newangle = angle - theta
    xform_newX = r * cosD(newangle) ' + x
    xform_newY = r * sinD(newangle) ' + y
End Sub

Function transform_result_x()
    transform_result_x = xform_newX
End Function

Function transform_result_y()
    transform_result_y = xform_newY
End Function


''' Math utilities

Function myMax(v1, V2)
    If v1 < V2 Then
        myMax = V2
    Else
        myMax = v1
    End If
End Function

Function myMin(v1, V2)
    If v1 < V2 Then
        myMin = v1
    Else
        myMin = V2
    End If
End Function

'''''''''''''''
''' Testing '''
'''''''''''''''

Sub testblock(xc, yc, theta)
    Dim i As Integer
    Dim Xb, Yb
    
    Call set_transform_values(xc, yc, theta)
    Xb = transform_result_x()
    Yb = transform_result_y()
    Box Xb, Yb, 0, -theta, 3, 7, 10
    
End Sub


Sub Analyze_Teleporters()
    Dim fnum As Integer
    Dim i As Integer
    Dim tName As String
    Dim tFid As Integer
    Dim tBid As Integer
    Dim tFront As String
    Dim tBack As String
    
    fnum = FreeFile
    Open "teleporters.txt" For Output As #fnum
    Print #fnum, "Teleporter Analysis"
    Print #fnum, "  Total: " & CStr(NumTeleporters)
    Print #fnum, " "
    Print #fnum, "  Breakdown:"
    Print #fnum, "Teleporter", "     ", "Front", "     ", "Back"
    For i = 1 To NumTeleporters
        tName = TeleporterName(i)
        tFid = TeleporterFront(i)
        tBid = TeleporterBack(i)
'        tFront = find_name_from_id(tFid)
'        tBack = find_name_from_id(tBid)
        Print #fnum, tName, "     ", CStr(tFid), "     ", CStr(tBid)
    Next i
    Close #fnum
End Sub

Function find_name_from_id(inID As Integer) As String
    Dim i As Integer
    Dim tstr As String
    For i = 1 To NumTeleporters
        If inID = TeleporterFront(i) Then
            tstr = TeleporterName(i) & " (front)"
            find_name_from_id = tstr
            Exit Function
        End If
        If inID = TeleporterBack(i) Then
            tstr = TeleporterName(i) & " (back)"
            find_name_from_id = tstr
            Exit Function
        End If
    Next i
    find_name_from_id = "[Unknown ID: " & CStr(inID) & "]"
End Function
