Attribute VB_Name = "Script_BorgCube"
'''''''''''''''''''''
'''''''''''''''''''''          Bryan A. Bentz
''' The Borg Cube '''          September 27, 2003
'''''''''''''''''''''
'''''''''''''''''''''

Const BorgVersion = 2

Const CubeCenterX = 0
Const CubeCenterY = 0
Const CubeEdge As Double = 400        'Total edge length
Const CubeHeight = 450
Const CubeAngle = 15        'Rotational angle (for interest)
Const CubeLevelHeight = 16                      'Floor (level) separation.
Const CubePlatformThickness = 1                 'Floor thickness

Const NumSquaresPerLevel = 10

Const SquareSize = 80
Const Minsize = 30
Const NumDiagonalsPerLevel = 3
Const DiagonalWidth = 5
Const DiagonalHeight = CubePlatformThickness

Const NumTeleportersPerLevel = 1
Const XWallSize = 40
Const NumXSquaresPerLevel = 20
Const YWallSize = 20
Const NumYSquaresPerLevel = 20

Const ProbabilityOfPyramidPerSquare = 0.2
Const PyramidSize = 12
Const PyramidHeight = 18

Const CubeXDistance = 30
Const CubeYDistance = 40

''' Derived values
Const CubeHalfEdge = CubeEdge / 2#
Const CubeCenterZ = CubeHeight + CubeHalfEdge
Const CubeLevels = CubeEdge / CubeLevelHeight   'Number of levels
Const CubeXPartitions = CubeEdge / CubeXDistance
Const CubeYPartitions = CubeEdge / CubeYDistance

Dim NumGroundTeleporters

Sub Write_Borg_MapFile()
    Dim i As Integer
    Dim j As Integer
    
    Dim CurHeight, curX, curY, curZ, xSize, ySize, zSize, Xb, Yb, zb

    Dim totalSize As Double
    Dim platsize As Double
    Dim percentage As Double
    
    MsgBox "Borg Cube may be version 1 or 2, set in code; running version " & CStr(BorgVersion)
    
    totalSize = CubeEdge * CubeEdge
    platsize = CDbl(SquareSize) * CDbl(SquareSize) * CInt(NumSquaresPerLevel) / 2 '2 because we use squaresize*rnd()
    percentage = (platsize / totalSize) * 100#
    'MsgBox "Percentage coverage per level is: " & Format(Percentage, "####0.##")
    
    
    
    '' Try to open the output file; exit if error.
    If Not Open_Output_File("borgmap" & CStr(version)) Then
        Exit Sub
    End If
    
     Init_Component_System
    
    define_component "CBOX"
            Dim sX, sY, sZ
            sX = 15
            sY = 10
            Box 0, 0, 0, 0, sX, sY, sZ
            Box -sX, 0, 0, 0, 1, sY, 1
            Box sX, 0, 0, 0, 1, sY, 1
            Box 0, sY, 0, 0, 1, sY, 1
            Box 0, -sY, 0, 0, 1, sY, 1
        end_component
    
    '' Header
    Write_Borg_Header
    
    ''
'''    'Do_Borg_Cube_Exterior
    curX = cubecenter - CubeHalfEdge
    curY = CubeCenterY - CubeHalfEdge
    curZ = CubeCenterZ - CubeHalfEdge
    Const PanelSize = 30
    Const PanelGap = 10 '50
    Const PanelThickness = 12
    Const TotalPanelSize = PanelSize + PanelGap
    Dim PanelCount, PanelX, PanelY, panelZ
    PanelCount = CubeEdge / TotalPanelSize
    For i = 1 To PanelCount
        For j = 1 To PanelCount
            PanelX = (i * TotalPanelSize) + CubeCenterX - CubeHalfEdge
            PanelY = (j * TotalPanelSize) + CubeCenterY - CubeHalfEdge

            ''XY plane faces (2 of them)
            Call set_transform_values(PanelX, PanelY, CubeAngle)
            Xb = transform_result_x()
            Yb = transform_result_y()
            Box Xb, Yb, CubeHeight, 90 - CubeAngle, PanelSize / 2, PanelSize / 2, PanelThickness
            Box Xb, Yb, CubeHeight + CubeEdge, 90 - CubeAngle, PanelSize / 2, PanelSize / 2, PanelThickness

            '' YZ plane faces (2 of them)
            panelZ = ((i - 1) * TotalPanelSize) + CubeHeight
            PanelX = -CubeHalfEdge
            'Use panelY from above
            Call set_transform_values(PanelX, PanelY, CubeAngle)
            Xb = transform_result_x()
            Yb = transform_result_y
            Box Xb + CubeCenterX, Yb + CubeCenterY, panelZ, 180 - CubeAngle, PanelThickness / 2, PanelSize / 2, PanelSize
            
            'Do it again with different X (ignore the symmetry optimization for now).
            PanelX = CubeHalfEdge
            Call set_transform_values(PanelX, PanelY, CubeAngle)
            Xb = transform_result_x()
            Yb = transform_result_y
            Box Xb + CubeCenterX, Yb + CubeCenterY, panelZ, _
              180 - CubeAngle, PanelThickness / 2, PanelSize / 2, PanelSize

            '' XZ plane faces (2 of them)
            PanelX = (i * TotalPanelSize) + CubeCenterX - CubeHalfEdge
            panelZ = ((j - 1) * TotalPanelSize) + CubeHeight
            PanelY = -CubeHalfEdge
            
            'Use panelY from above
            Call set_transform_values(PanelX, PanelY, CubeAngle)
            Xb = transform_result_x()
            Yb = transform_result_y
            Box Xb + CubeCenterX, Yb + CubeCenterY, panelZ, 180 - CubeAngle, PanelSize / 2, PanelThickness / 2, PanelSize
            
            'Do it again with different X (ignore the symmetry optimization for now).
            PanelY = CubeHalfEdge
            Call set_transform_values(PanelX, PanelY, CubeAngle)
            Xb = transform_result_x()
            Yb = transform_result_y
            Box Xb + CubeCenterX, Yb + CubeCenterY, panelZ, _
              180 - CubeAngle, PanelSize / 2, PanelThickness / 2, PanelSize


        Next j
    Next i


    'Do_Borg_Groundstuff
    Const NumEdgePartitions = 3
    For i = 1 To NumEdgePartitions
        For j = 1 To NumEdgePartitions
            'Insert teleporter
            curX = i * (800 / NumEdgePartitions) - 400
            curY = j * (800 / NumEdgePartitions) - 400
                Teleporter curX - (400 / NumEdgePartitions), curY - (400 / NumEdgePartitions), 0, 360 * Rnd(), 0.45, 10, 20, default_bsize, "Ground " & CStr(i) & CStr(j)
            'include limiting walls, so combat is only in the cube
            'X wall
            'Box x, y, z, rotation, sX, sY, sZ
            If i < NumEdgePartitions Then
                Box curX, 0, 0, 0, 10, 400, 20
            End If
            If j < NumEdgePartitions Then
                Box 0, curY, 0, 0, 400, 10, 20
            End If
        Next j
    Next i
    NumGroundTeleporters = NumTeleporters

    '' Loop through each floor
    Dim CubeLowX, CubeHighX, CubeLowY, CubeHighY, CubeLowZ, CubeHighZ
    CubeLowX = CubeCenterX - CubeHalfEdge
    CubeHighX = CubeCenterX + CubeHalfEdge
    CubeLowY = CubeCenterY - CubeHalfEdge
    CubeHighY = CubeCenterY + CubeHalfEdge
   ' CubeLowZ = 0 ' 0 - CubeHalfEdge
    'CubeHighZ = CubeEdge '0 + CubeHalfEdge
    CubeLowZ = CubeHeight
    CubeHighZ = CubeLowZ + CubeEdge

    Dim global_curZ
    For i = 1 To CubeLevels
        curZ = i * CubeLevelHeight + CubeHeight
        global_curZ = curZ
        
        If BorgVersion = 1 Then
    '        '' loop through the number of platforms
            For j = 1 To NumSquaresPerLevel
                curZ = global_curZ
                curX = CubeEdge * Rnd() - CubeHalfEdge
                curY = CubeEdge * Rnd() - CubeHalfEdge
    
                'Make a guess at some sizes;
                xSize = myMax(SquareSize * Rnd(), Minsize)
                ySize = myMax(SquareSize * Rnd(), Minsize)
    
                'Make sure we're not going beyond the edges of the cube.
                Dim lowx, highx, lowy, highy, lowz, highz
                lowx = curX - xSize
                highx = curX + xSize
                lowy = curY - ySize
                highy = curY + hSize
                lowz = curZ
                highz = lowz + CubePlatformThickness
    
                If lowx < CubeLowX Then
                    curX = curX + Abs(CubeLowX - lowx)
                End If
                If highx > CubeHighX Then
                    curX = curX - Abs(CubeHighX - highx)
                End If
                If lowy < CubeLowY Then
                    curY = curY + Abs(CubeLowY - lowy)
                End If
                If highy > CubeHighY Then
                    curY = curY - Abs(CubeHighY - highy)
                End If
                If lowz < CubeLowZ Then
                    curZ = curZ + Abs(CubeLowZ - lowz)
                End If
                If highz > CubeHighZ Then
                    curZ = curZ - Abs(CubeHighZ - highz)
                End If
    
                Call set_transform_values(curX, curY, CubeAngle)
                Xb = transform_result_x()
                Yb = transform_result_y()
                Box Xb, Yb, curZ, 90 - CubeAngle, xSize, ySize, CubePlatformThickness
    
                If Rnd() < ProbabilityOfPyramidPerSquare Then
                    Pyramid Xb, Yb, curZ + CubePlatformThickness, 360 * Rnd(), _
                    2 + PyramidSize * Rnd(), _
                    2 + PyramidSize * Rnd(), _
                    PyramidHeight * Rnd()
                End If
            Next j
        Else
            'Borg version 2
            'Do X/Y box
            curZ = global_curZ
            
            curX = 0
            curY = CubeHalfEdge
            xSize = CubeHalfEdge - 16
            ySize = 16
            Call set_transform_values(curX, curY, CubeAngle)
            Xb = transform_result_x()
            Yb = transform_result_y()
            Box Xb, Yb, curZ, -CubeAngle, xSize, ySize, CubePlatformThickness
            
            curX = 0
            curY = -CubeHalfEdge + 16
            xSize = CubeHalfEdge
            ySize = 16
            Call set_transform_values(curX, curY, CubeAngle)
            Xb = transform_result_x()
            Yb = transform_result_y()
            Box Xb, Yb, curZ, -CubeAngle, xSize, ySize, CubePlatformThickness
                      
            curX = CubeHalfEdge - 16
            curY = 0
            xSize = 16 'CubeHalfEdge
            ySize = CubeHalfEdge
            Call set_transform_values(curX, curY, CubeAngle)
            Xb = transform_result_x()
            Yb = transform_result_y()
            Box Xb, Yb, curZ, -CubeAngle, xSize, ySize, CubePlatformThickness
            
            curX = -CubeHalfEdge + 16
            curY = 0
            xSize = 16 'CubeHalfEdge
            ySize = CubeHalfEdge
            Call set_transform_values(curX, curY, CubeAngle)
            Xb = transform_result_x()
            Yb = transform_result_y()
            Box Xb, Yb, curZ, -CubeAngle, xSize, ySize, CubePlatformThickness
           
            
        End If
        '' loop through the number of transporters per level.
        For j = 1 To NumTeleportersPerLevel
            ''Teleporter platforms are 5 high; if cubeplatformthickness
            '' is different, correct so that the driving surfaces match.
            curZ = global_curZ - (5 - CubePlatformThickness)

            curX = (CubeHalfEdge / 2# - 20) * Rnd() + CubeHalfEdge / 2#
            curY = (CubeHalfEdge / 2# - 20) * Rnd() + CubeHalfEdge / 2#
            If Rnd() > 0.5 Then curX = -curX
            If Rnd() > 0.5 Then curY = -curY

'            lowx = curX - 10
'            highx = curX + 10
'            lowy = curY - 10
'            highy = curY + 10
'
'            If lowx < CubeLowX Then
'                curX = curX + Abs(CubeLowX - lowx)
'            End If
'            If highx > CubeHighX Then
'                curX = curX - Abs(CubeHighX - highx)
'            End If
'            If lowy < CubeLowY Then
'                curY = curY + Abs(CubeLowY - lowy)
'            End If
'            If highy > CubeHighY Then
'                curY = curY - Abs(CubeHighY - highy)
'            End If
            Call set_transform_values(curX, curY, CubeAngle)
            Xb = transform_result_x()
            Yb = transform_result_y()
            teleporter_platform Xb, Yb, curZ, 90 - CubeAngle
        Next j


        If BorgVersion = 1 Then
            '' loop through the number of diagonal ramps per level.
            For j = 1 To NumDiagonalsPerLevel
                curZ = global_curZ
                Dim curx1, curx2, cury1, cury2
                curx1 = CubeEdge * Rnd() - CubeHalfEdge
                cury1 = CubeEdge * Rnd() - CubeHalfEdge
                curx2 = CubeEdge * Rnd() - CubeHalfEdge
                cury2 = CubeEdge * Rnd() - CubeHalfEdge
    
                lowx = curx1 - 10
                highx = curx1 + 10
                lowy = cury1 - 10
                highy = cury1 + 10
    
                'X1, Y1 check
                If lowx1 < CubeLowX Then
                    curx1 = curx1 + Abs(CubeLowX - lowx1)
                End If
                If highx1 > CubeHighX Then
                    curx1 = curx1 - Abs(CubeHighX - highx1)
                End If
                If lowy1 < CubeLowY Then
                    cury1 = cury1 + Abs(CubeLowY - lowy1)
                End If
                If highy1 > CubeHighY Then
                    cury1 = cury1 - Abs(CubeHighY - highy1)
                End If
    
                'X2, Y2 check
                lowx = curx2 - 10
                highx = curx2 + 10
                lowy = cury2 - 10
                highy = cury2 + 10
    
                If lowx < CubeLowX Then
                    curx2 = curx2 + Abs(CubeLowX - lowx)
                End If
                If highx > CubeHighX Then
                    curx2 = curx2 - Abs(CubeHighX - highx)
                End If
                If lowy < CubeLowY Then
                    cury2 = cury2 + Abs(CubeLowY - lowy)
                End If
                If highy > CubeHighY Then
                    cury2 = cury2 - Abs(CubeHighY - highy)
                End If
    
                Call set_transform_values(curX, curY, CubeAngle)
                Xb = transform_result_x()
                Yb = transform_result_y()
                straight_wall curx1, cury1, curx2, cury2, curZ, DiagonalWidth, DiagonalHeight, 0, False, 0, 0, 0, 0
            Next j
        Else
            'Borg Version 2.0
            'For this level, pick 4 possible diagonals,
            'and run them probabilistically.
            Dim xd1 As Double
            Dim xd2 As Double
            Dim yd1 As Double
            Dim yd2 As Double
            Dim xfd1, xfd2, yfd1, yfd2
            xd1 = (CubeHalfEdge - 16) * 1
            If Rnd() > 0.5 Then xd1 = -xd1
            yd1 = Rnd() * (CubeHalfEdge - 16)
            If Rnd() > 0.5 Then yd1 = -yd1
            
            xd2 = 0
            yd2 = 0
'            xd2 = -xd1
'            If Rnd() > 0.5 Then xd2 = -xd2
'            yd2 = Rnd() * (CubeHalfEdge - 16)
'            If Rnd() > 0.5 Then yd2 = -yd2

'            If Rnd() > 0.5 Then
'                'reverse x, y coords
'                Dim oldx1, oldy1, oldx2, oldy2
'                oldx1 = xd1
'                oldx2 = xd2
'                oldy1 = yd1
'                oldy2 = yd2
'                xd1 = oldy1
'                xd2 = oldy2
'                yd1 = oldx1
'                yd2 = oldx2
'            End If

            set_transform_values xd1, yd1, CubeAngle
            xfd1 = transform_result_x()
            yfd1 = transform_result_y()
            set_transform_values yd1, yd2, CubeAngle
            xfd2 = transform_result_x()
            yfd2 = transform_result_y()

            xfd2 = 0
            yfd2 = 0
            'Now, draw a block between these two points
            BoxConnect xfd1, yfd1, xfd2, yfd2, curZ, 10, CubePlatformThickness
            BoxConnect xfd2, yfd2, -xfd1, -yfd1, curZ, 10, CubePlatformThickness
            
           
                
        End If
    Next i

    ''Enterprises!
    NCC_1701 -100, -100, CubeHeight - 100, 45
    For i = 1 To 3
        NCC_1701 RandomAroundCube(), RandomAroundCube(), _
            Rnd() * CubeEdge + CubeHeight, 360 * Rnd()
    Next i


    '' Now do X, Y vertical partitions
    If BorgVersion = 1 Then
        'X
        For i = 1 To CubeXPartitions
            curX = i * CubeXDistance - CubeHalfEdge
    
            '' loop through the number of walls
            For j = 1 To NumXSquaresPerLevel
                curY = CubeEdge * Rnd() - CubeHalfEdge
                curZ = CubeEdge * Rnd() + CubeHeight
                xSize = CubePlatformThickness / 2#
                ySize = myMax(XWallSize * Rnd(), 2)
                zSize = myMax(XWallSize * Rnd(), 2)
    
                'Make sure we're not going beyond the edges of the cube.
                lowz = curZ - zSize
                highz = curZ + zSize
                lowy = curY - ySize
                highy = curY + ySize
    
                If lowz < CubeLowZ Then
                    curZ = curZ + Abs(CubeLowZ - lowz)
                End If
                If highz > CubeHighZ Then
                    curZ = curZ - Abs(CubeHighZ - highz)
                End If
                If lowy < CubeLowY Then
                    curY = curY + Abs(CubeLowY - lowy)
                End If
                If highy > CubeHighY Then
                    curY = curY - Abs(CubeHighY - highy)
                End If
                Call set_transform_values(curX, curY, CubeAngle)
                Xb = transform_result_x()
                Yb = transform_result_y()
                Box Xb, Yb, curZ, 90 - CubeAngle, xSize, ySize, zSize
            Next j
        Next i
    '
    '    '' Now, by Y
        For i = 1 To CubeYPartitions
            curY = i * CubeYDistance - CubeHalfEdge
    
            '' loop through the number of walls
            For j = 1 To NumYSquaresPerLevel
                curX = CubeEdge * Rnd() - CubeHalfEdge
                curZ = CubeEdge * Rnd() + CubeHeight
    
                'Make sure we're not going beyond the edges of the cube.
                ySize = CubePlatformThickness / 2#
                xSize = myMax(XWallSize * Rnd(), 2)
                zSize = myMax(XWallSize * Rnd(), 2)
    
                lowz = curZ - zSize
                highz = curZ + zSize
                lowy = curY - ySize
                highy = curY + ySize
    
                If lowz < CubeLowZ Then
                    curZ = curZ + Abs(CubeLowZ - lowz)
                End If
                If highz > CubeHighZ Then
                    curZ = curZ - Abs(CubeHighZ - highz)
                End If
                If lowx < CubeLowX Then
                    curX = curX + Abs(CubeLowX - lowx)
                End If
                If highx > CubeHighX Then
                    curX = curX - Abs(CubeHighX - highx)
                End If
    
                Call set_transform_values(curX, curY, CubeAngle)
                Xb = transform_result_x()
                Yb = transform_result_y()
                Box Xb, Yb, curZ, 90 - CubeAngle, xSize, ySize, zSize
            Next j
        Next i
    Else
        'BorgVersion 2
        Const bRadius = 150
        Dim theta, zradius
        Dim phi, circleX, circleY, circleZ

        
        For theta = 0 To 180 Step 20
            For phi = 0 To 360 Step 20
                zradius = bRadius * cosD(phi)
                circleX = zradius * cosD(theta)
                circleY = zradius * sinD(theta)
                circleZ = bRadius * sinD(phi) + CubeCenterZ
                            
                Box circleX, circleY, circleZ, theta, 15, 10, 4
                Box circleX, circleY, circleZ + 4, theta, 5, 4, 12
                Pyramid circleX, circleY, circleZ + 16, theta, 2, 2, 5, False, True
                'Place_Component "CBOX", x, y, z, theta
            Next phi
        Next theta
        
        Dim sizeX
        For i = 1 To 5
            
            Xb = i * 50 - 125
            Yb = -100
    '        Call set_transform_values(Xb, Yb, CubeAngle)
    '        Xb = transform_result_x()
    '        Yb = transform_result_y()
            Box Xb, Yb, CubeCenterZ - CubeHalfEdge, -CubeAngle + 90, 5, 10, 200
            
            
            Yb = 100
            Box Xb, Yb, CubeCenterZ - CubeHalfEdge, -CubeAngle + 90, 5, 10, 200
        Next i
    End If
'
getout:
    '' Connect teleporters
    '     Ground teleporters go to random cube teleporters
    '     Cube teleporters to to random teleporters (some ground?  For now.)
    
    Dim GuessTP As Integer
    For i = 1 To NumGroundTeleporters
guessloop:
        GuessTP = (NumTeleporters + 1) * Rnd()
        If GuessTP <= NumGroundTeleporters Then GoTo guessloop
        
        'We can connect it.
        makelink TeleporterFront(i), TeleporterFront(GuessTP)
        
        ''Do this again (not elegant, but hey.)
guessloop2:
        GuessTP = (NumTeleporters + 1) * Rnd()
        If GuessTP <= NumGroundTeleporters Then GoTo guessloop2
        
        makelink TeleporterBack(i), TeleporterBack(GuessTP)
    Next i
    
    For i = NumGroundTeleporters To NumTeleporters
        GuessTP = (NumTeleporters + 1) * Rnd()
        makelink TeleporterFront(i), TeleporterFront(GuessTP)
        GuessTP = (NumTeleporters + 1) * Rnd()
        makelink TeleporterBack(i), TeleporterBack(GuessTP)
    Next i
    
  Close_Output_File
            
End Sub



Sub Write_Borg_Header()
    comment "  THE BORG CUBE, version " & CStr(BorgVersion)
    comment "   "
    comment "   Version 1:  September 27, 2003"
    comment "   Version 2:  " & CStr(Date)
    comment "   "
    comment "     by "
    comment "   "
    comment "       Bryan A. Bentz"
    comment "       a.k.a. ""The Purple Panzer"""
    comment "       Email:"
    comment "            bentz@bentz-engineering.com, or "
    comment "            bentz@ai.mit.edu"
    comment "       URLs:"
    comment "            Personal URL: http://www-swiss.ai.mit.edu/~bentz/"
    comment "            Business URL: http://www.bentz-engineering.com/"
    comment "   "
    comment "   "
    If BorgVersion = 1 Then
        comment "  This is my second map, and the idea is to construct a floating"
        comment "  playable cube.  I want to do this quickly, so it will be almost"
        comment "  completely autogenerated (albeit with use of RND(), so every run"
        comment "  will be different).  Not sure if it'll be decent playing, but I"
        comment "  expect that it probably will be - a bit more 3-dimensional than"
        comment "  most people are used to."
        comment " "
    End If
End Sub

Function RandomAroundCube()
    Dim minX, maxX, xrange, sign
    If Rnd() > 0.5 Then
        sign = -1
    Else
        sign = 1
    End If
    minX = CubeHalfEdge
    maxX = 380
    xrange = maxX - minX
    RandomAroundCube = (xrange * Rnd() + minX) * sign
End Function
