Attribute VB_Name = "Script_HugeMap"
Const NumTowns = 6
Const WorldSize = 2000
Dim TownNames(NumTowns) As String
Dim GlobalX As Long
Dim GlobalY As Long

Sub Write_HugeMap()

        
        Dim i As Integer
        Dim x(NumTowns) As Long
        Dim y(NumTowns) As Long
        Dim Xp As Long
        Dim Yp As Long
        
        If Not Open_Output_File("HugeMap") Then
            Exit Sub
        End If
        
        Init_Component_System
        
        comment "Experiment in Huge Map (Version 2)"
        comment "   - The Purple Panzer"
        comment "3/19/2005"
        comment " "
        comment "Parser, code support system by"
        comment " "
        comment " Bryan A. Bentz"
        comment "       a.k.a. ""The Purple Panzer"""
        comment "       Email:"
        comment "            bentz@bentz-engineering.com, or "
        comment "            bentz@; ai.mit.edu; "
        comment "       URLs:"
        comment "            Personal URL: http://www-swiss.ai.mit.edu/~bentz/"
        comment "            Business URL: http://www.bentz-engineering.com/"
        comment " "
        comment " "
        
        world (WorldSize)
        
        highway_physics 1, 0, 0
        
        'Init random number generator for repeatability
        Rnd (-10)
        Randomize
        Rnd (32)
        
        'Assign town name
        TownNames(1) = "Fargo"
        TownNames(2) = "Silver City"
        TownNames(3) = "Sioux City"
        TownNames(4) = "Carson City"
        TownNames(5) = "Telluride"
        TownNames(6) = "Tombstone"
        For i = 1 To NumTowns
            
            'Determine position
            Call find_position(i)
            x(i) = GlobalX
            y(i) = GlobalY
            build_town i, x(i), y(i)
        Next i
        
        For i = 1 To NumTowns
            'Central square teleporter
            Teleporter x(i), y(i), 0, 0, 0.45, 10, 12, 1, TownNames(i)
        Next i
        
        'Connect teleporters
        For i = 1 To NumTowns - 1
            make_teleporter_link TownNames(i), 1, TownNames(i + 1), 2
            make_teleporter_link TownNames(i + 1), 2, TownNames(i), 1
        Next i
        make_teleporter_link TownNames(NumTowns), 1, TownNames(1), 2
        make_teleporter_link TownNames(1), 2, TownNames(NumTowns), 1
        
        
        For i = 1 To 30
            Xp = ABSrnd(WorldSize)
            Yp = ABSrnd(WorldSize)
            Pyramid Xp, Yp, 0, 360 * Rnd(), 40, 40, 40, False, False
        Next i
        For i = 1 To 20
            Xp = ABSrnd(WorldSize)
            Yp = ABSrnd(WorldSize)
            Pyramid Xp, Yp, 0, 360 * Rnd(), 40, 40, 15, False, False, True
        Next i
       Close_Output_File
End Sub

Sub build_town(index As Integer, Xc As Long, Yc As Long)
    Const AvenueWidth = 22
    Const StreetWidth = 14
    Const Blocksize = 30
    
    Dim CentralSquareSize As Integer
    Dim i As Integer
    Dim theta As Long
    Dim iBlock As Integer
    Dim jBlock As Integer
    Dim NumBlocks As Integer
    Dim BuildingsPerBlock As Integer
    Dim Xb As Long
    Dim Yb As Long
    
    NumBlocks = Abs(ABSrnd(3, 2))
    CentralSquareSize = Blocksize + NumBlocks * 8
    
    Debug.Print "NUMBLOCKS: " & CStr(NumBlocks)
    
    define_component CStr(TownNames(index))
        
        For iBlock = -NumBlocks To NumBlocks
            For jBlock = -NumBlocks To NumBlocks
                Xb = iBlock * (Blocksize + AvenueWidth)
                Yb = jBlock * (Blocksize + StreetWidth)
                If Abs(Xb) >= CentralSquareSize Or Abs(Yb) >= CentralSquareSize Then
                    'Toss coin to randomly insert building
                    If Rnd() < 0.85 Then
                        Insert_Block Xb, Yb, Blocksize / 2#
                    End If
                    'Throw in pyramids in intersections to break things up
                    If iBlock < NumBlocks And jBlock < NumBlocks Then
                        Pyramid Xb + Blocksize / 2# + AvenueWidth / 2#, Yb + Blocksize / 2# + StreetWidth / 2#, 0, 45, 2, 2, 10, False, True, False
                    End If
                End If
                
            Next jBlock
        Next iBlock
    end_component
    theta = 360 * Rnd()
    Place_Component CStr(TownNames(index)), Xc, Yc, 0, theta
End Sub


Sub Insert_Block(Xb, Yb, Blocksize)
'    BuildingsPerBlock = 4 * Rnd() + 2
'    For i = 1 To BuildingsPerBlock
        'insert_building iblock*blocksize, jblock*blocksize, blocksize,
    Insert_Building Xb, Yb, Blocksize
End Sub

Sub Insert_Building(Xb, Yb, BuildingSize)
    Dim zSize As Long
    Dim hSize As Long
    Dim baseSize As Long
    Dim CurrentHeight As Long
    
    Debug.Print "Insert Building at (" & CStr(Xb) & ", " & CStr(Yb) & ") size: " & CStr(BuildingSize)
    zSize = 30 * Rnd() + 5
    baseSize = zSize
    Box Xb, Yb, 0, 0, BuildingSize, BuildingSize, zSize
    CurrentHeight = zSize
    
    'randomly do second story
    If Rnd() > 0.6 Then
        zSize = 15 * Rnd() + 3
        hSize = Rnd() * (baseSize - 5) + 5
        Box Xb, Yb, CurrentHeight, 0, hSize, hSize, zSize
        CurrentHeight = CurrentHeight + zSize
        baseSize = hSize
    End If
    
    'randomly do pyramid top
    If Rnd() > 0.7 Then
        zSize = 10 * Rnd() + 5
        hSize = Rnd * (baseSize - 2) + 2
        Pyramid Xb, Yb, CurrentHeight, 0, hSize, hSize, zSize, False, True
    End If
End Sub

Sub find_position(i As Integer)
    'set GlobalX, GlobalY
    Dim vThird As Double
    Dim hQuarter As Double
    vThird = WorldSize / 3#
    hQuarter = WorldSize / 4#
    
    Select Case i
        Case 1
            GlobalX = hQuarter * 0.5
            GlobalY = vThird * 0.5
        Case 2
            GlobalX = hQuarter * 3.5
            GlobalY = vThird * 0.5
            
        Case 3
            GlobalX = hQuarter * 1.5
            GlobalY = vThird * 1.2
        Case 4
            GlobalX = hQuarter * 2.7
            GlobalY = vThird * 1.5
            
        Case 5
            GlobalX = hQuarter * 0.5
            GlobalY = vThird * 2.5
        Case 6
            GlobalX = hQuarter * 3.5
            GlobalY = vThird * 2.5
        Case Else
            GlobalX = WorldSize * Rnd()
            GlobalY = WorldSize * Rnd()
            If Rnd() > 0.5 Then GlobalX = -GlobalX
            If Rnd() > 0.5 Then GlobalY = -GlobalY
    End Select
        GlobalX = GlobalX - WorldSize / 2#
        GlobalY = GlobalY - WorldSize / 2#
        GlobalX = 2 * GlobalX
        GlobalY = 2 * GlobalY
        
End Sub

Function floor(numerator, denominator) As Double
    Dim factor As Integer
    factor = numerator / denominator
    
End Function

        
